import type { AxisScale } from 'd3-axis';
import type { ScaleOrdinal } from 'd3-scale';
import { CSSProperties, ReactElement, ReactNode } from 'react';
export declare type Horizontal = 'left' | 'right';
export declare type Vertical = 'top' | 'bottom';
export declare type Margins = Record<Horizontal | Vertical, number | undefined>;
export interface Series {
    x: number[];
    y: number[];
}
export interface PlotProps {
    width: number;
    height: number;
    colorScheme?: string[];
    margin?: Margins;
    children: ReactNode[];
}
export interface LineSeriesProps {
    data: Series;
    lineStyle?: CSSProperties;
    label?: string;
    displayMarker?: boolean;
    markerShape?: 'circle' | 'square' | 'triangle';
    markerSize?: number;
}
interface AxisProps {
    fontSize?: number;
    label?: string;
    showGridLines?: boolean;
    labelStyle?: CSSProperties;
    labelSpace?: number;
    min?: number;
    max?: number;
}
export interface XAxisProps extends AxisProps {
    paddingLeft?: number;
    paddingRight?: number;
}
export interface YAxisProps extends AxisProps {
    paddingTop?: number;
    paddingBottom?: number;
}
export interface HeadingProps {
    title: string;
    titleStyle?: CSSProperties;
    titleClass?: string;
    subtitle?: string;
    subtitleStyle?: CSSProperties;
    subtitleClass?: string;
    position?: Vertical;
}
export interface LegendProps {
    position?: Horizontal;
}
export interface MarkersProps {
    x: number;
    y: number;
    fill: string;
    size: number;
}
export interface SeriesType {
    id: string;
    xMin: number;
    xMax: number;
    yMin: number;
    yMax: number;
    label: string;
}
export interface PlotContextType {
    xScale?: AxisScale<number>;
    yScale?: AxisScale<number>;
    width?: number;
    height?: number;
    left?: number;
    right?: number;
    top?: number;
    bottom?: number;
    plotWidth?: number;
    plotHeight?: number;
    labels?: string[];
    colorScaler?: ScaleOrdinal<string, string>;
    xScientific: boolean;
    yScientific: boolean;
}
export declare type ReducerActions = {
    type: 'newData';
    value: SeriesType;
} | {
    type: 'removeData';
    value: {
        id: string;
    };
} | {
    type: 'xMinMax' | 'yMinMax';
    value: {
        min?: number;
        max?: number;
    };
} | {
    type: 'xPadding' | 'yPadding';
    value: {
        min?: number;
        max?: number;
    };
};
export interface PlotChildren {
    invalidChild: boolean;
    lineSeries: ReactElement<LineSeriesProps>[];
    axis: Record<'x' | 'y', ReactElement<AxisProps> | null>;
    heading: ReactElement<HeadingProps> | null;
    legend: ReactElement<LegendProps> | null;
}
export {};
