"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const d3_array_1 = require("d3-array");
const d3_scale_1 = require("d3-scale");
const d3_scale_chromatic_1 = require("d3-scale-chromatic");
const react_1 = __importStar(require("react"));
const hooks_1 = require("./hooks");
const utils_1 = require("./utils");
function reducer(state, action) {
    switch (action.type) {
        case 'newData': {
            const { series } = state;
            const serieItem = action.value;
            return { ...state, series: [...series, serieItem] };
        }
        case 'removeData': {
            const { id } = action.value;
            const seriesFiltered = state.series.filter((series) => series.id !== id);
            return { ...state, series: seriesFiltered };
        }
        case 'xMinMax': {
            const { min: xMin, max: xMax } = action.value;
            return { ...state, xMin, xMax };
        }
        case 'yMinMax': {
            const { min: yMin, max: yMax } = action.value;
            return { ...state, yMin, yMax };
        }
        case 'xPadding': {
            const { min: paddingLeft, max: paddingRight } = action.value;
            return { ...state, paddingLeft, paddingRight };
        }
        case 'yPadding': {
            const { min: paddingBottom, max: paddingTop } = action.value;
            return { ...state, paddingBottom, paddingTop };
        }
        default: {
            throw new Error();
        }
    }
}
function Plot({ width, height, margin, colorScheme, children, }) {
    const [state, dispatch] = react_1.useReducer(reducer, { series: [] }, undefined);
    const compoundComp = utils_1.splitChildren(children);
    const { invalidChild, lineSeries, axis, heading, legend } = compoundComp;
    if (invalidChild) {
        throw new Error('Only compound components of Plot are displayed');
    }
    // Distances in plot
    const { left = 0, right = 0, top = 0, bottom = 0 } = margin;
    const plotWidth = width - left - right;
    const plotHeight = height - top - bottom;
    // Set scales
    const { xScale, xMin, xMax } = react_1.useMemo(() => {
        const xMin = state.xMin !== undefined ? state.xMin : d3_array_1.min(state.series, (d) => d.xMin);
        const xMax = state.xMax !== undefined ? state.xMax : d3_array_1.max(state.series, (d) => d.xMax);
        if ([xMin, xMax].includes(undefined))
            return {};
        if (xMin > xMax) {
            throw new Error(`X: min (${xMin}) is bigger than max (${xMax})`);
        }
        const leftPad = (xMax - xMin) * (state.paddingLeft || 0);
        const rightPad = (xMax - xMin) * (state.paddingRight || 0);
        return {
            xMin,
            xMax,
            xScale: d3_scale_1.scaleLinear()
                .domain([xMin - leftPad, xMax + rightPad])
                .range([left, width - right]),
        };
    }, [state, left, right, width]);
    const { yScale, yMin, yMax } = react_1.useMemo(() => {
        // Get limits from state or data
        const yMin = state.yMin !== undefined ? state.yMin : d3_array_1.min(state.series, (d) => d.yMin);
        const yMax = state.yMax !== undefined ? state.yMax : d3_array_1.max(state.series, (d) => d.yMax);
        // Limits validation
        if ([yMin, yMax].includes(undefined))
            return {};
        if (yMin > yMax) {
            throw new Error(`Y: min (${yMin}) is bigger than max (${yMax})`);
        }
        // Limits paddings
        const topPad = (yMax - yMin) * (state.paddingTop || 0);
        const bottomPad = (yMax - yMin) * (state.paddingBottom || 0);
        return {
            yMin,
            yMax,
            yScale: d3_scale_1.scaleLinear()
                .domain([yMin - bottomPad, yMax + topPad])
                .range([height - bottom, top]),
        };
    }, [state, bottom, top, height]);
    const labels = state.series.map(({ label }) => label);
    const colorScaler = react_1.useMemo(() => d3_scale_1.scaleOrdinal()
        .range(colorScheme || d3_scale_chromatic_1.schemeSet1)
        .domain(labels), [colorScheme, labels]);
    const xDiff = xMax - xMin;
    const yDiff = yMax - yMin;
    return (react_1.default.createElement(hooks_1.PlotContext.Provider, { value: {
            xScale,
            yScale,
            width,
            height,
            left,
            right,
            top,
            bottom,
            plotWidth,
            plotHeight,
            labels,
            colorScaler,
            xScientific: xDiff <= 0.01 || xDiff >= 1000,
            yScientific: yDiff <= 0.01 || yDiff >= 1000,
        } },
        react_1.default.createElement(hooks_1.DispatchContext.Provider, { value: { dispatch } },
            react_1.default.createElement("svg", { width: width, height: height, style: { fontFamily: 'Arial, Helvetica, sans-serif' } },
                heading,
                legend,
                axis.x,
                axis.y,
                react_1.default.createElement("clipPath", { id: "squareClip" },
                    react_1.default.createElement("rect", { fillOpacity: "0", x: left, y: top, width: plotWidth, height: plotHeight })),
                react_1.default.createElement("g", { style: { clipPath: 'url(#squareClip)' } }, lineSeries)))));
}
exports.default = Plot;
