"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const d3_axis_1 = require("d3-axis");
const d3_selection_1 = require("d3-selection");
const react_1 = __importStar(require("react"));
const hooks_1 = require("../hooks");
const XAxis = ({ label, fontSize = 16, labelSpace = 20, showGridLines, labelStyle, min, max, paddingLeft = 0, paddingRight = 0, }) => {
    const axisRef = react_1.useRef(null);
    const { xScale, xScientific, plotHeight, plotWidth, bottom, left, height, width, } = hooks_1.usePlotContext();
    const { dispatch } = hooks_1.useDispatchContext();
    // Send min and max to main state
    react_1.useEffect(() => {
        dispatch({ type: 'xMinMax', value: { min, max } });
    }, [dispatch, min, max]);
    // Send paddings to main state
    react_1.useEffect(() => {
        if (paddingLeft < 0 || paddingLeft > 1) {
            throw new Error(`Padding left (${paddingLeft}) is not between 0 and 1`);
        }
        if (paddingRight < 0 || paddingRight > 1) {
            throw new Error(`Padding right (${paddingRight}) is not between 0 and 1`);
        }
        dispatch({
            type: 'xPadding',
            value: { min: paddingLeft, max: paddingRight },
        });
    }, [dispatch, paddingLeft, paddingRight]);
    react_1.useEffect(() => {
        if ((axisRef === null || axisRef === void 0 ? void 0 : axisRef.current) && xScale) {
            const axis = d3_axis_1.axisBottom(xScale);
            if (showGridLines) {
                axis.tickSizeInner(-plotHeight);
            }
            if (xScientific) {
                axis.tickFormat((val) => val.toExponential(2));
            }
            d3_selection_1.select(axisRef.current)
                .call(axis)
                .call((g) => {
                g.selectAll('.tick line')
                    .attr('stroke-opacity', showGridLines ? 0.5 : 1)
                    .attr('stroke-dasharray', showGridLines ? '2,2' : '0');
                g.selectAll('.tick text')
                    .attr('transform', `translate(0,${showGridLines ? 6 : 0})`)
                    .style('user-select', 'none');
                if (xScientific) {
                    g.selectAll('.tick:nth-child(odd) text').style('display', 'none');
                }
            });
        }
    }, [axisRef, xScale, plotHeight, height, width, showGridLines, xScientific]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement("g", { ref: axisRef, transform: `translate(0, ${height - bottom})` }),
        label && (react_1.default.createElement("text", { x: plotWidth / 2 + left, y: height - bottom + labelSpace + fontSize, fontSize: fontSize, textAnchor: "middle", style: labelStyle }, label))));
};
exports.default = XAxis;
